@@ -138,6 +138,97 @@ fn gather_slurm_jobs() -> Result<Vec<usize>,std::io::Error>
 	).collect())
 }
 
+fn slurm_get_association(field:&str) -> Result<String,()>
+{
+	let command = Command::new("sacctmgr")
+		.arg("list")
+		.arg("associations")
+		.arg("-p")
+		.output().map_err(|_|())?;
+	let output=String::from_utf8_lossy(&command.stdout);
+	let mut lines = output.lines();
+	//let (index,header) = lines.next().unwrap().split('|').enumerate().find(|i,ifield|ifield==field).unwrap_or_else(||format!("field {} not found in header"));
+	let mut index_user=0;
+	let mut index_field=0;
+	let header = lines.next().ok_or( () )?;
+	for (header_index,header_field) in header.split('|').enumerate()
+	{
+		if header_field == "User"
+		{
+			index_user=header_index;
+		}
+		if header_field == field
+		{
+			index_field =header_index;
+		}
+	}
+	//let user = std::env::var("USER").unwrap_or_else(|_|panic!("could not read $USER"));
+	let user = std::env::var("USER").map_err( |_|() )?;
+	for line in lines
+	{
+		let values:Vec<&str> = line.split('|').collect();
+		if values[index_user]==user
+		{
+			return Ok(values[index_field].to_string());
+		}
+	}
+	//panic!("field not found");
+	Err( () )
+}
+
+fn slurm_get_qos(name:&str, field:&str) -> Result<String,()>
+{
+	//sacctmgr show qos -p
+	let command=Command::new("sacctmgr")
+		.arg("show")
+		.arg("qos")
+		.arg("-p")
+		.output().map_err(|_|())?;
+	let output=String::from_utf8_lossy(&command.stdout);
+	let mut lines = output.lines();
+	//Name==main -> MaxSubmitPU?->value
+	let mut index_name=0;
+	let mut index_field=0;
+	let header = lines.next().ok_or( () )?;
+	for (header_index,header_field) in header.split('|').enumerate()
+	{
+		if header_field == "Name"
+		{
+			index_name=header_index;
+		}
+		if header_field == field
+		{
+			index_field =header_index;
+		}
+	}
+	for line in lines
+	{
+		let values:Vec<&str> = line.split('|').collect();
+		if values[index_name]==name
+		{
+			return Ok(values[index_field].to_string());
+		}
+	}
+	//panic!("field not found");
+	Err( () )
+}
+
+pub fn slurm_available_space() -> Result<usize,()>
+{
+	let command=Command::new("squeue")
+		.arg("-ho")
+		.arg("%A")
+		.arg("--me")
+		.output().map_err(|_|())?;
+	let output=String::from_utf8_lossy(&command.stdout);
+	let current = output.lines().count();
+	let qos = slurm_get_association("Def QOS")?;//--> main ?
+	let maximum = slurm_get_qos(&qos,"MaxSubmitPU")?;//--> 2000 ?
+	//let maximum = maximum.parse::<usize>().expect("should be an integer");
+	let maximum = maximum.parse::<usize>().map_err(|_|())?;
+	Ok(maximum - current)
+}
+
 ///Simulations to be run in a slurm/other job.
 struct Job
 {
@@ -313,14 +404,20 @@ pub struct ExperimentFiles
 
 impl ExperimentFiles
 {
-	pub fn build_cfg_contents(&mut self)
+	/// panic indicates whether to allow panic in this code. Otherwise do not build the content and be silent.
+	pub fn build_cfg_contents(&mut self, panic:bool)
 	{
 		if let None = self.cfg_contents
 		{
 			let cfg=self.root.as_ref().unwrap().join("main.cfg");
 			if let Some(session) = &self.ssh2_session {
-				let sftp = session.sftp().expect("error starting sftp");
-				let mut remote_main_cfg =  sftp.open(&cfg).expect("Could not open remote main.cfg");
+				//let sftp = session.sftp().expect("error starting sftp");
+				let sftp = session.sftp();
+				if !panic && sftp.is_err() { return; }
+				let sftp = sftp.expect("error starting sftp");
+				let remote_main_cfg =  sftp.open(&cfg);
+				if !panic && remote_main_cfg.is_err() { return; }
+				let mut remote_main_cfg= remote_main_cfg.expect("Could not open remote main.cfg");
 				let mut remote_main_cfg_contents=String::new();
 				remote_main_cfg.read_to_string(&mut remote_main_cfg_contents).expect("Could not read remote main.cfg.");
 				self.cfg_contents = Some(remote_main_cfg_contents);
@@ -354,7 +451,7 @@ impl ExperimentFiles
 	{
 		if let None = self.parsed_cfg
 		{
-			self.build_cfg_contents();
+			self.build_cfg_contents(false);
 			let parsed_cfg=match config_parser::parse(self.cfg_contents_ref())
 			{
 				Err(x) => panic!("error parsing configuration file: {:?}",x),
@@ -938,6 +1035,15 @@ impl<'a> Experiment<'a>
 					slurm_time=value.to_string();
 				}
 				slurm_mem=mem.map(|x:&str|x.to_string());
+				// TODO: we can look how many jobs we can submit by
+				// $ sacctmgr list user $USER
+				// $ sacctmgr list associations
+				// $ sacctmgr show qos
+				//as described in https://stackoverflow.com/questions/61565703/get-maximum-number-of-jobs-allowed-in-slurm-cluster-as-a-user
+				if let Ok(available) = slurm_available_space()
+				{
+					println!("Available number of jobs to send to slurm is {}",available);
+				}
 			},
 			Action::Check =>
 			{
@@ -946,7 +1052,7 @@ impl<'a> Experiment<'a>
 			Action::Pull =>
 			{
 				self.initialize_remote();
-				self.remote_files.as_mut().unwrap().build_cfg_contents();
+				self.remote_files.as_mut().unwrap().build_cfg_contents(true);
 				self.files.compare_cfg(&self.remote_files.as_ref().unwrap());
 			},
 			Action::RemoteCheck =>
@@ -991,7 +1097,7 @@ impl<'a> Experiment<'a>
 					},
 				};
 				//check remote config
-				self.remote_files.as_mut().unwrap().build_cfg_contents();
+				self.remote_files.as_mut().unwrap().build_cfg_contents(false);
 				if self.remote_files.as_ref().unwrap().cfg_enough_content() {
 					self.files.compare_cfg(&self.remote_files.as_ref().unwrap());
 				} else {
@@ -1491,7 +1597,7 @@ impl<'a> Experiment<'a>
 				Err(x) => panic!("error parsing output description file: {:?}",x),
 				Ok(config_parser::Token::Value(ConfigurationValue::Array(ref descriptions))) => for description in descriptions.iter()
 				{
-					println!("description={}",description);
+					//println!("description={}",description);
 					match create_output(&description,&results,self.files.experiments.len(),&self.files)
 					{
 						Ok(_) => (),
